@@ -1,5 +1,5 @@
 #####
-# Copyright (c) 2011, NVIDIA Corporation.  All rights reserved.
+# Copyright (c) 2011-2012, NVIDIA Corporation.  All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -116,6 +116,15 @@ NVML_LED_COLOR_AMBER        = 1
 nvmlFlagDefault             = 0
 nvmlFlagForce               = 1
 
+# buffer size
+NVML_DEVICE_INFOROM_VERSION_BUFFER_SIZE      = 16
+NVML_DEVICE_UUID_BUFFER_SIZE                 = 80
+NVML_SYSTEM_DRIVER_VERSION_BUFFER_SIZE       = 81
+NVML_SYSTEM_NVML_VERSION_BUFFER_SIZE         = 80
+NVML_DEVICE_NAME_BUFFER_SIZE                 = 64
+NVML_DEVICE_SERIAL_BUFFER_SIZE               = 30
+NVML_DEVICE_VBIOS_VERSION_BUFFER_SIZE        = 32
+
 NVML_VALUE_NOT_AVAILABLE_ulonglong = c_ulonglong(-1)
 
 ## Lib loading ##
@@ -146,7 +155,7 @@ def _nvmlGetFunctionPointer(name):
             raise NVMLError(NVML_ERROR_UNINITIALIZED)
         try:
             return getattr(nvmlLib, name)
-        except AttributeError, attrError:
+        except AttributeError as attrError:
             raise NVMLError(NVML_ERROR_NOT_SUPPORTED)
     finally:
         # lock is always freed
@@ -331,7 +340,7 @@ def nvmlInit():
                     else:
                         # assume linux
                         nvmlLib = CDLL("libnvidia-ml.so")
-                except OSError, ose:
+                except OSError as ose:
                     print(ose)
                     _nvmlCheckReturn(NVML_ERROR_DRIVER_NOT_LOADED)
                 if (nvmlLib == None):
@@ -367,9 +376,9 @@ def nvmlErrorString(result):
 
 # Added in 2.285
 def nvmlSystemGetNVMLVersion():
-    c_version = create_string_buffer(256)
+    c_version = create_string_buffer(NVML_SYSTEM_NVML_VERSION_BUFFER_SIZE)
     fn = _nvmlGetFunctionPointer("nvmlSystemGetNVMLVersion")
-    ret = fn(c_version, c_uint(256))
+    ret = fn(c_version, c_uint(NVML_SYSTEM_NVML_VERSION_BUFFER_SIZE))
     _nvmlCheckReturn(ret)
     return c_version.value
 
@@ -382,9 +391,9 @@ def nvmlSystemGetProcessName(pid):
     return c_name.value
 
 def nvmlSystemGetDriverVersion():
-    c_version = create_string_buffer(100)
+    c_version = create_string_buffer(NVML_SYSTEM_DRIVER_VERSION_BUFFER_SIZE)
     fn = _nvmlGetFunctionPointer("nvmlSystemGetDriverVersion")
-    ret = fn(c_version, c_uint(100))
+    ret = fn(c_version, c_uint(NVML_SYSTEM_DRIVER_VERSION_BUFFER_SIZE))
     _nvmlCheckReturn(ret)
     return c_version.value
 
@@ -506,6 +515,14 @@ def nvmlDeviceGetHandleBySerial(serial):
     ret = fn(c_serial, byref(device))
     _nvmlCheckReturn(ret)
     return device
+
+def nvmlDeviceGetHandleByUUID(uuid):
+    c_uuid = c_char_p(uuid)
+    device = c_nvmlDevice_t()
+    fn = _nvmlGetFunctionPointer("nvmlDeviceGetHandleByUUID")
+    ret = fn(c_uuid, byref(device))
+    _nvmlCheckReturn(ret)
+    return device
     
 def nvmlDeviceGetHandleByPciBusId(pciBusId):
     c_busId = c_char_p(pciBusId)
@@ -516,31 +533,31 @@ def nvmlDeviceGetHandleByPciBusId(pciBusId):
     return device
 
 def nvmlDeviceGetName(handle):
-    c_name = create_string_buffer(64)
+    c_name = create_string_buffer(NVML_DEVICE_NAME_BUFFER_SIZE)
     fn = _nvmlGetFunctionPointer("nvmlDeviceGetName")
-    ret = fn(handle, c_name, c_uint(64))
+    ret = fn(handle, c_name, c_uint(NVML_DEVICE_NAME_BUFFER_SIZE))
     _nvmlCheckReturn(ret)
     return c_name.value
     
 def nvmlDeviceGetSerial(handle):
-    c_serial = create_string_buffer(30)
+    c_serial = create_string_buffer(NVML_DEVICE_SERIAL_BUFFER_SIZE)
     fn = _nvmlGetFunctionPointer("nvmlDeviceGetSerial")
-    ret = fn(handle, c_serial, c_uint(30))
+    ret = fn(handle, c_serial, c_uint(NVML_DEVICE_SERIAL_BUFFER_SIZE))
     _nvmlCheckReturn(ret)
     return c_serial.value
     
 def nvmlDeviceGetUUID(handle):
-    c_uuid = create_string_buffer(80)
+    c_uuid = create_string_buffer(NVML_DEVICE_UUID_BUFFER_SIZE)
     fn = _nvmlGetFunctionPointer("nvmlDeviceGetUUID")
-    ret = fn(handle, c_uuid, c_uint(80))
+    ret = fn(handle, c_uuid, c_uint(NVML_DEVICE_UUID_BUFFER_SIZE))
     _nvmlCheckReturn(ret)
     return c_uuid.value
     
 def nvmlDeviceGetInforomVersion(handle, infoRomObject):
-    c_version = create_string_buffer(16)
+    c_version = create_string_buffer(NVML_DEVICE_INFOROM_VERSION_BUFFER_SIZE)
     fn = _nvmlGetFunctionPointer("nvmlDeviceGetInforomVersion")
     ret = fn(handle, _nvmlInforomObject_t(infoRomObject),
-	         c_version, c_uint(16))
+	         c_version, c_uint(NVML_DEVICE_INFOROM_VERSION_BUFFER_SIZE))
     _nvmlCheckReturn(ret)
     return c_version.value
     
@@ -701,9 +718,9 @@ def nvmlDeviceGetPendingDriverModel(handle):
 
 # Added in 2.285
 def nvmlDeviceGetVbiosVersion(handle):
-    c_version = create_string_buffer(32)
+    c_version = create_string_buffer(NVML_DEVICE_VBIOS_VERSION_BUFFER_SIZE)
     fn = _nvmlGetFunctionPointer("nvmlDeviceGetVbiosVersion")
-    ret = fn(handle, c_version, c_uint(32))
+    ret = fn(handle, c_version, c_uint(NVML_DEVICE_VBIOS_VERSION_BUFFER_SIZE))
     _nvmlCheckReturn(ret)
     return c_version.value
 
@@ -818,4 +835,69 @@ def nvmlEventSetFree(eventSet):
     _nvmlCheckReturn(ret)
     return None
 
+# Added in 2.285
+def nvmlEventDataGetPerformanceState(data):
+    fn = _nvmlGetFunctionPointer("nvmlEventDataGetPerformanceState")
+    pstate = _nvmlPstates_t()
+    ret = fn(byref(data), byref(pstate))
+    _nvmlCheckReturn(ret)
+    return pstate.value
+
+# Added in 2.285
+def nvmlEventDataGetXidCriticalError(data):
+    fn = _nvmlGetFunctionPointer("nvmlEventDataGetXidCriticalError")
+    xid = c_uint()
+    ret = fn(byref(data), byref(xid))
+    _nvmlCheckReturn(ret)
+    return xid.value
+
+# Added in 2.285
+def nvmlEventDataGetEccErrorCount(data):
+    fn = _nvmlGetFunctionPointer("nvmlEventDataGetEccErrorCount")
+    ecc = c_ulonglong()
+    ret = fn(byref(data), byref(ecc))
+    _nvmlCheckReturn(ret)
+    return ecc.value
+
+# Added in 3.295
+def nvmlDeviceOnSameBoard(handle1, handle2):
+    fn = _nvmlGetFunctionPointer("nvmlDeviceOnSameBoard")
+    onSameBoard = c_int()
+    ret = fn(handle1, handle2, byref(onSameBoard))
+    _nvmlCheckReturn(ret)
+    return (onSameBoard.value != 0)
+
+# Added in 3.295
+def nvmlDeviceGetCurrPcieLinkGeneration(handle):
+    fn = _nvmlGetFunctionPointer("nvmlDeviceGetCurrPcieLinkGeneration")
+    gen = c_uint()
+    ret = fn(handle, byref(gen))
+    _nvmlCheckReturn(ret)
+    return gen.value
+
+# Added in 3.295
+def nvmlDeviceGetMaxPcieLinkGeneration(handle):
+    fn = _nvmlGetFunctionPointer("nvmlDeviceGetMaxPcieLinkGeneration")
+    gen = c_uint()
+    ret = fn(handle, byref(gen))
+    _nvmlCheckReturn(ret)
+    return gen.value
+
+# Added in 3.295
+def nvmlDeviceGetCurrPcieLinkWidth(handle):
+    fn = _nvmlGetFunctionPointer("nvmlDeviceGetCurrPcieLinkWidth")
+    width = c_uint()
+    ret = fn(handle, byref(width))
+    _nvmlCheckReturn(ret)
+    return width.value
+
+# Added in 3.295
+def nvmlDeviceGetMaxPcieLinkWidth(handle):
+    fn = _nvmlGetFunctionPointer("nvmlDeviceGetMaxPcieLinkWidth")
+    width = c_uint()
+    ret = fn(handle, byref(width))
+    _nvmlCheckReturn(ret)
+    return width.value
+
+
 
